import { Container, Point, Text, TextStyle } from 'pixi.js'
import { createText, timerStyle } from './utilities/fonts'

import { gsap } from 'gsap'
import { Timer } from './utilities/Timer'

export class TimerText extends Container {
  private timer: Timer
  private text?: Text
  animation: boolean = false
  textStyle: TextStyle = timerStyle()
  private countdown: number = 0
  private isInitiated: boolean = false

  constructor(
    private seconds: number,
    private point: Point,
  ) {
    super()
    this.timer = new Timer(seconds)
    this.timer.on('timeout', this.onTImeOut.bind(this))
    this.timer.on('tick', this.onTick.bind(this))
    this.countdown = seconds
    this.render()
  }

  private onTImeOut() {
    this.timer.reset()
    this.emit('timeout')
  }

  private onTick(seconds: number) {
    !this.animation && this.text?.destroy()
    this.countdown = seconds
    this.render()
  }

  reset() {
    this.timer.stop()
    this.timer.reset()
    this.countdown = this.seconds
    this.isInitiated = false
    this.render()
  }

  start() {
    this.timer.reset()
    this.timer.start()
  }

  private render() {
    if (!this.animation) this.removeChildren()
    this.text = createText({
      text: this.countdown.toString(),
      x: this.point.x,
      y: this.point.y,
      style: this.textStyle,
      container: this,
    })
    this.text.scale.set(1)
    if (this.animation && this.isInitiated) this.animate(this.text)
  }

  private animate(text: Text) {
    if (text) {
      gsap.timeline().to(text, {
        delay: 0.5,
        duration: 1.2,
        x: this.point.x + 2,
        y: this.point.y - 25,
        ease: 'power4.out',
        alpha: 0.0,
        onComplete: () => {
          text.destroy()
        },
        // scale: 1,
      })
      // gsap.to(this.text, {
      //   duration: 1,
      //   alpha: 0,
      //   x: this.app.canvas.width / 2,
      //   y: this.app.canvas.height / 2 - 20,
      //   scale: 1.3,
      //   ease: 'linear',
      // })
    }
  }
}
